import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import java.util.*;

// FEATURE: check google's 3D warehouse for sample shapes

public class Main implements GLEventListener, MouseListener, MouseMotionListener, KeyListener, MouseWheelListener {
    
    boolean alt = false;
    boolean shift = false;
    boolean control = false;

    public void mouseWheelMoved(MouseWheelEvent e) {
        tz -= e.getWheelRotation() ;
    }

    public void keyTyped(KeyEvent e)  { }
    public void keyPressed(KeyEvent e)  {
        switch(e.getKeyCode()) {
            case KeyEvent.VK_CONTROL: control = true; break;
            case KeyEvent.VK_ALT: alt = true; break;
            case KeyEvent.VK_SHIFT: shift = true; break;
        }
    }
    public void keyReleased(KeyEvent e) {
        switch(e.getKeyCode()) {
            case KeyEvent.VK_CONTROL: control = false; break;
            case KeyEvent.VK_ALT: alt = false; break;
            case KeyEvent.VK_SHIFT: shift = false; break;
        }
    }

    public void mouseClicked(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
    public void mousePressed(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) { }

    int mousex;
    int mousey;
    public void mouseMoved(MouseEvent e) {
        mousex = e.getX();
        mousey = e.getY();
    }

    float tx = 0;
    float ty = 0;
    float tz = 0;
    float anglex = 0;
    float angley = 0;
    public void mouseDragged(MouseEvent e) {
        if (shift) {
            tx += (mousex - e.getX())/(float)20;
            ty += (mousey - e.getY())/(float)20;
        } else {
            anglex -= mousex - e.getX();
            angley += mousey - e.getY();
        }
        mousex = e.getX();
        mousey = e.getY();
    }

    private Geom geom = new Geom();

    /** magnification factor */
    private static final float MAG = 1;

    Geom.V[] translations;
    Geom.P[] points;

    public Main(StlFile stlf) {

        Geom.P ltf = geom.newP(-0.2,  0.1,  0.1);
        Geom.P mtf = geom.newP( 0.0,  0.1,  0.1);
        Geom.P rtf = geom.newP( 0.2,  0.1,  0.1);
        Geom.P ltn = geom.newP(-0.2,  0.1, -0.1);
        Geom.P mtn = geom.newP( 0.0,  0.1, -0.1);
        Geom.P rtn = geom.newP( 0.2,  0.1, -0.1);
        Geom.P lbf = geom.newP(-0.2, -0.1,  0.1);
        Geom.P mbf = geom.newP( 0.0, -0.1,  0.1);
        Geom.P rbf = geom.newP( 0.2, -0.1,  0.1);
        Geom.P lbn = geom.newP(-0.2, -0.1, -0.1);
        Geom.P mbn = geom.newP( 0.0, -0.1, -0.1);
        Geom.P rbn = geom.newP( 0.2, -0.1, -0.1);
        
        points = new Geom.P[] {
            ltf,
            mtf,
            rtf,
            ltn,
            mtn,
            rtn,
            lbf,
            mbf,
            rbf,
            lbn,
            mbn,
            rbn
        };

        translations = new Geom.V[] {
            geom.new V(-0.2,  0.2,    0),
            geom.new V( 0.2,  0.2,    0),
            geom.new V(-0.2, -0.2,    0),
            geom.new V( 0.2, -0.2,    0),
            geom.new V( 0.4,    0,    0),
            geom.new V(-0.4,    0,    0),
            geom.new V(   0,    0,  0.2),
            geom.new V(   0,    0, -0.2),
        };

        // top
        geom.newT(ltf, mtf, mtn);
        geom.newT(mtn, ltn, ltf);
        geom.newT(mtf, rtf, rtn);
        geom.newT(rtn, mtn, mtf);

        // bottom (swap normals)
        geom.newT(mbf, lbf, mbn);
        geom.newT(lbn, mbn, lbf);
        geom.newT(rbf, mbf, rbn);
        geom.newT(mbn, rbn, mbf);
        
        // left
        geom.newT(ltf, ltn, lbn);
        geom.newT(lbn, lbf, ltf);

        // right (swap normals)
        geom.newT(rtn, rtf, rbn);
        geom.newT(rbf, rbn, rtf);

        // front
        geom.newT(ltn, mtn, mbn);
        geom.newT(ltn, mbn, lbn);
        geom.newT(mtn, rtn, rbn);
        geom.newT(mtn, rbn, mbn);

        // back
        geom.newT(mtf, ltf, mbf);
        geom.newT(mbf, ltf, lbf);
        geom.newT(rtf, mtf, rbf);
        geom.newT(rbf, mtf, mbf);

        for(Geom.V v : translations) {
            for(Geom.T t1 : geom) {
                for(Geom.T t2 : geom) {
                    if (t1==t2) continue;

                    if ((t1.p1().plus(v).minus(t2.p1()).mag() < Geom.EPSILON) &&
                        (t1.p2().plus(v).minus(t2.p3()).mag() < Geom.EPSILON) &&
                        (t1.p3().plus(v).minus(t2.p2()).mag() < Geom.EPSILON)) {
                        t1.e1().bind(t2.e3().pair);
                        t1.e2().bind(t2.e2().pair);
                        t1.e3().bind(t2.e1().pair);
                    }
                    if ((t1.p2().plus(v).minus(t2.p1()).mag() < Geom.EPSILON) &&
                        (t1.p3().plus(v).minus(t2.p3()).mag() < Geom.EPSILON) &&
                        (t1.p1().plus(v).minus(t2.p2()).mag() < Geom.EPSILON)) {
                        t1.e2().bind(t2.e3().pair);
                        t1.e3().bind(t2.e2().pair);
                        t1.e1().bind(t2.e1().pair);
                    }
                    if ((t1.p3().plus(v).minus(t2.p1()).mag() < Geom.EPSILON) &&
                        (t1.p1().plus(v).minus(t2.p3()).mag() < Geom.EPSILON) &&
                        (t1.p2().plus(v).minus(t2.p2()).mag() < Geom.EPSILON)) {
                        t1.e3().bind(t2.e3().pair);
                        t1.e1().bind(t2.e2().pair);
                        t1.e2().bind(t2.e1().pair);
                    }
                }
            }
        }

        Geom.P mid = ltn.getE(mbn).shatter();

        //tx.e2.shatter();
        //tx.e3.shatter();


        geom.bind();

        mid.move(geom.new V((float)0,0,(float)-0.05));
        //ltn.move(geom.new V((float)0,0,(float)-0.05));

        //mtf.move(geom.new V(0, (float)-0.05, (float)0.05));
        /*
        for(int i=0; i<100; i++) {
            rand();
        }
        */


        /*
        for(int i=0; i<stlf.coordArray.length; i+=3) {
            Geom.P p0 = geom.newP(stlf.coordArray[i+0].x * MAG, stlf.coordArray[i+0].y * MAG, stlf.coordArray[i+0].z * MAG);
            Geom.P p1 = geom.newP(stlf.coordArray[i+1].x * MAG, stlf.coordArray[i+1].y * MAG, stlf.coordArray[i+1].z * MAG);
            Geom.P p2 = geom.newP(stlf.coordArray[i+2].x * MAG, stlf.coordArray[i+2].y * MAG, stlf.coordArray[i+2].z * MAG);
            Geom.V n  = geom.new V(stlf.normArray[i/3].x * MAG, stlf.normArray[i/3].y  * MAG, stlf.normArray[i/3].z * MAG);
            Geom.T t  = geom.newT(p0, p1, p2, n);
        }
        */


    }
        Random random = new Random();

    public synchronized void breakit() {
        int i = Math.abs(random.nextInt()) % geom.es.size();
        Geom.E e = geom.es.poll();
        System.out.println("shatter " + e);
        e.shatter();
        geom.unbind();
        geom.bind();
    }
    public synchronized void rand() {
        for(int i=0; i<10; i++) {
            Geom.P p = geom.ts.get(Math.abs(random.nextInt()) % geom.ts.size()).e1().p1;
            float r1 = Math.abs(random.nextFloat());
            r1 = r1 - (float)Math.floor(r1);
            r1 = r1 * (float)0.01;
            r1 = r1 - (float)0.005;
            switch(Math.abs(random.nextInt()) % 3) {
                case 0: p.move(geom.new V((float)0,  (float)r1, (float)0)); break;
                case 1: p.move(geom.new V((float)r1, (float)0,  (float)0)); break;
                case 2: p.move(geom.new V((float)0,  (float)0, (float)r1)); break;
            }
        }
    }

    /**
     * Take care of initialization here.
     */
    public void init(GLAutoDrawable gld) {
        GL gl = gld.getGL();
        gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
        gl.glViewport(0, 0, 500, 300);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glClearDepth(1.0);
        gl.glDepthFunc(GL.GL_LEQUAL);
        gl.glLoadIdentity();
        //glu.gluOrtho2D(0.0, 500.0, 0.0, 300.0);
        display(gld);
    }

    public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) { }
    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) { }
    public synchronized void display(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();
        GLU glu = new GLU();
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        gl.glPointSize(5.0f);
        gl.glLoadIdentity();
        glu.gluPerspective(50-tz, ((float)drawable.getWidth())/drawable.getHeight(), 0.5, 10);
        glu.gluLookAt(0, 0, -1, 0, 0, 0, 0, 1, 0);
        gl.glTranslatef(tx/(float)20, ty/(float)20, 0);
        gl.glRotatef(anglex/3, 0, 1, 0);
        gl.glRotatef(angley/3, 1, 0, 0);

        gl.glBegin(GL.GL_TRIANGLES);
        draw(gl, true);
        gl.glEnd();

        for(Geom.V v1 : translations) {
            if (v1.z==0 && v1.y==0) continue;
            Geom.V v = v1.times((float)1.1);
            gl.glTranslatef(v.x, v.y, v.z);
            draw(gl, false);
            gl.glTranslatef(-v.x, -v.y, -v.z);
            break;
        }

    }

    private synchronized void draw(GL gl, boolean triangles) {
        float red = 0.0f;
        float green = 0.0f;
        float blue = 0.0f;
        for(Geom.T t : geom) {
            if (red < 0.15) red = 1.0f;
            if (green < 0.15) green = 1.0f;
            if (blue < 0.15) blue = 1.0f;
            red -= .09f;
            green -= .12f;
            blue -= .15f;
            gl.glColor3f(red, green, blue);
            switch(t.color) {
                case 0: gl.glColor3f((float)0.25, (float)0.25, (float)0.75); break;
                case 1: gl.glColor3f((float)0.25, (float)0.75, (float)0.25); break;
                case 2: gl.glColor3f((float)0.75, (float)0.25, (float)0.25); break;
                case 3: gl.glColor3f((float)0.50, (float)0.50, (float)0.50); break;
            }
            //gl.glBegin(GL.GL_LINES);
            if (triangles) {
                gl.glBegin(GL.GL_TRIANGLES);
                t.glVertices(gl);
                gl.glEnd();
            } else {
                gl.glBegin(GL.GL_LINES);
                t.e1().p1.glVertex(gl);
                t.e1().p2.glVertex(gl);
                t.e2().p1.glVertex(gl);
                t.e2().p2.glVertex(gl);
                t.e3().p1.glVertex(gl);
                t.e3().p2.glVertex(gl);
                gl.glEnd();
            }

            Geom.P centroid = t.centroid();
            gl.glBegin(GL.GL_LINES);
            gl.glColor3f(1, 1, 1);
            /*
            centroid.glVertex(gl);
            centroid.plus(t.norm().times(t.diameter())).glVertex(gl);
            */
            /*
            t.p1().glVertex(gl);
            t.p1().plus(t.p1().norm().times(t.diameter())).glVertex(gl);
            t.p2().glVertex(gl);
            t.p2().plus(t.p2().norm().times(t.diameter())).glVertex(gl);
            t.p3().glVertex(gl);
            t.p3().plus(t.p3().norm().times(t.diameter())).glVertex(gl);
            */
            gl.glEnd();

        }
    }

    public static void main(String[] s) throws Exception {
        StlFile stlf = new StlFile();
        stlf.load("teapot.stl");
        Main main = new Main(stlf);
        Frame f = new Frame();
        GLCapabilities glcaps = new GLCapabilities();
        GLCanvas glcanvas = new GLCanvas();
        glcanvas.addGLEventListener(main);
        f.add(glcanvas, BorderLayout.CENTER);
        f.pack();
        f.show();
        f.setSize(900, 900);
        f.doLayout();

        glcanvas.addMouseListener(main);
        glcanvas.addMouseMotionListener(main);
        glcanvas.addMouseWheelListener(main);
        glcanvas.addKeyListener(main);

        int i = 0;
        while(true) {
            Thread.sleep(10);
            glcanvas.repaint();
            main.rand();
            if (i++>10) { main.breakit(); i = 0; }
       }

    }

}